# SPDX-License-Identifier: WTFPL
# SPDX-FileCopyrightText: 2023 Anna <cyber@sysrq.in>
# No warranty

""" Common data for Python generators """

BUG_TRACKER_LABELS = ["bug tracker", "bugtracker", "bugs", "issues"]
CHANGELOG_LABELS = ["changelog", "changes"]
DOCS_LABELS = ["doc", "docs", "documentation"]
HOME_REPO_LABELS = ["source", "repo", "repository", "home", "homepage"]
